@@ -48,6 +48,7 @@ import { NotebookAdapter, NotebookButtons } from './adapters/notebook';
 import {
   Dialog,
   ICommandPalette,
+  InputDialog,
   showDialog,
   showErrorMessage
 } from '@jupyterlab/apputils';
@@ -69,6 +70,9 @@ import { refreshIcon } from '@jupyterlab/ui-components';
 import OutputMode = CiteProc.OutputMode;
 import { cite2cPlugin } from './formats/cite2c';
 import { markdownDOIPlugin } from './formats/markdownDOI';
+import getItem = InputDialog.getItem;
+import { NameVariable } from './_csl_data';
+import { migrationDialog } from './components/dialogs';
 
 const PLUGIN_ID = 'jupyterlab-citation-manager:plugin';
 
@@ -245,24 +249,14 @@ class UnifiedCitationManager implements ICitationManager {
         detectionResult.citationsDetected !== 0 ||
         detectionResult.bibliographiesDetected !== 0
       ) {
-        const migrateButton = Dialog.okButton({
-          label: this.trans.__('Migrate')
-        });
-        const decision = await showDialog({
-          title: this.trans.__(
-            'Importing %1 citations is possible',
-            format.name
-          ),
-          body: this.trans.__(
-            'Detected %1 citation in %2 format. Would you like to migrate these citations to Citation Manager format?',
-            detectionResult.citationsDetected,
-            format.name
-          ),
-          buttons: [Dialog.cancelButton(), migrateButton],
-          defaultButton: 1
-        });
-        if (decision.button === migrateButton) {
-          const result = await format.migrateFrom(panel, adapter);
+        const decision = await migrationDialog(
+          format,
+          detectionResult,
+          panel.context.path,
+          this.trans
+        );
+        if (decision.button.accept) {
+          const result = await format.migrateFrom(panel, adapter, this);
           if (result.aborted) {
             await showErrorMessage(
               this.trans.__('Migration from %1 failed', format.name),
@@ -296,15 +290,19 @@ class UnifiedCitationManager implements ICitationManager {
   ): Promise<IStylePreview> {
     if (!this.notebookTracker.currentWidget) {
       throw {
-        reason:
+        reason: this.trans.__(
           'Please switch to a document that supports citations to generate a preview'
+        )
       } as IPreviewNotAvailable;
     }
     const panel = this.notebookTracker.currentWidget;
+    // TODO: it would be good to show citation clusters if present
     const citations = this.getAdapter(panel).citations.slice(0, maxCitations);
     if (citations.length === 0) {
       throw {
-        reason: 'No citations in the document to generate the preview from.'
+        reason: this.trans.__(
+          'No citations in the document to generate the preview from.'
+        )
       } as IPreviewNotAvailable;
     }
     const processor = await this.createProcessor(style.id);
@@ -344,6 +342,126 @@ class UnifiedCitationManager implements ICitationManager {
     }
   }
 
+  /**
+   * Given citable data, find the best match by comparing:
+   * - identifier (`id`)
+   * - digital object identifier (`DOI`)
+   * - unified resource identifier (`URL)
+   * - title and surnames of all authors
+   * with citable items available from each of the registered providers.
+   *
+   * If multiple matches are found the user
+   * will be prompted to resolve the ambiguity by selecting
+   * which of the match to use, displaying the message given
+   * in `context` argument.
+   *
+   * If titles of works matched by ID or DOI do not match,
+   * a dialog asking for confirmation will be displayed.
+   */
+  async matchItem(
+    data: Partial<ICitableData>,
+    context: string
+  ): Promise<IUnambiguousItemIdentifier | null> {
+    interface IMatch {
+      identifier: IUnambiguousItemIdentifier;
+      item: ICitableData;
+    }
+    const surnamesAsString = (authors: NameVariable[]) => {
+      return authors.map(author => author?.given).join(', ');
+    };
+    const compareTitles = (a: string, b: string) => {
+      return a.trim().toLocaleLowerCase() === b.trim().toLocaleLowerCase();
+    };
+    const dataAuthors = data.author
+      ? surnamesAsString(data.author)
+      : this.trans.__('unknown authors');
+    const matches: IMatch[] = [];
+    for (const provider of this.providers.values()) {
+      await provider.isReady;
+      for (const item of provider.citableItems.values()) {
+        const isMatch =
+          (data.id && item.id === data.id) ||
+          (data.DOI && item.DOI === data.DOI) ||
+          (data.URL && item.URL === data.URL) ||
+          (data.title &&
+            item.title &&
+            compareTitles(item.title, data.title) &&
+            data.author &&
+            item.author &&
+            surnamesAsString(item.author) === surnamesAsString(data.author));
+        if (isMatch) {
+          matches.push({
+            identifier: {
+              source: provider.id,
+              id: item.id as string
+            },
+            item: item
+          });
+        }
+      }
+    }
+    if (matches.length > 1) {
+      const choices = matches.map(match => {
+        return match.item.author
+          ? this.trans.__(
+              '"%1" by "%2" from %3',
+              match.item.title,
+              surnamesAsString(match.item.author),
+              match.identifier.source
+            )
+          : this.trans.__(
+              '"%1" from %3',
+              match.item.title,
+              match.identifier.source
+            );
+      });
+      const result = await getItem({
+        items: choices,
+        title: this.trans.__('Matching references'),
+        label: this.trans.__(
+          'Please choose the best matching item for "%1" by "%2" in order to %2',
+          data.title,
+          dataAuthors,
+          context
+        )
+      });
+      if (result.value === null) {
+        return null;
+      }
+      return matches[choices.indexOf(result.value)].identifier;
+    }
+    if (matches.length === 1) {
+      const match = matches[0];
+      if (
+        match.item.title &&
+        data.title &&
+        !compareTitles(match.item.title, data.title)
+      ) {
+        const result = await showDialog({
+          title: this.trans.__('Matching references'),
+          body: this.trans.__(
+            'The only matching references for "%1" by "%2" has a different title: "%2"',
+            data.title,
+            dataAuthors,
+            match.item.title
+          ),
+          buttons: [
+            Dialog.cancelButton(),
+            Dialog.createButton({
+              label: this.trans.__('Accept match anyway'),
+              accept: true
+            })
+          ]
+        });
+        if (!result.button.accept) {
+          return null;
+        }
+      }
+      return match.identifier;
+    }
+    return null;
+  }
+
   protected async processFromScratch(
     panel: NotebookPanel,
     styleId: string | undefined = undefined
@@ -351,7 +469,7 @@ class UnifiedCitationManager implements ICitationManager {
     const progressBase: Partial<IProgress> = {
       label: this.trans.__('Updating citations'),
       tooltip: this.trans.__(
-        'Citation manager is updating citations and bibliography...'
+        'Citation manager is updating citations and bibliography…'
       )
     };
     this.progress.emit({ ...progressBase, state: 'started' });
@@ -928,7 +1046,7 @@ const managerPlugin: JupyterFrontEndPlugin<ICitationManager> = {
     try {
       referenceBrowser.id = 'jupyterlab-citation-manager:reference-browser';
       referenceBrowser.title.icon = bookshelfIcon;
-      referenceBrowser.title.caption = 'Reference Browser';
+      referenceBrowser.title.caption = trans.__('Reference Browser');
       referenceBrowser.show();
       // below the git extension but not at the very end
       app.shell.add(referenceBrowser, 'left', { rank: 850 });
